#!/usr/local/bin/wish
# GUI for the Xplanets simulation
#
# GUI designed by Christopher White
# for the LCS&E Summer High School Outreach Program
# Email: white@lcse.umn.edu
#
# Planet Simulation written by Paul Woodward
# Email: paul@lcse.umn.edu
#
# Modifications to program done by Rich Solstad
# Email: rich@lcse.umn.edu

##########################################################################
# This procedure displays the "About" information
##########################################################################
proc About { message ysize} {
    toplevel .about
    wm transient .about .
    
    wm geometry .about 350x$ysize+300+300
    
    message .about.t -relief sunken -font 8x16 \
            -width 350 -text $message
    
    button .about.b1 -text "Ok" -command {destroy .about}
    
    pack .about.t
    pack .about.b1 -fill y -padx 1m -pady 1m
    
    update idletask
    grab .about
    tkwait window .about
}

##########################################################################
# This procedure sends xplanest the "KILL" signal to kill it
##########################################################################
proc Stop-Planets { } {
    set username [exec whoami]
    catch { set killwhat [exec ps -u $username | grep xplanets] }
    catch { exec kill -KILL $killwhat}
    catch { file delete planet-input }
    destroy .controls
}

##########################################################################
# This procedure sends xplanets the "STOP" signal to pause it
##########################################################################
proc Pause-Planets { } {
    set username [exec whoami]
    catch { set killwhat [exec ps -u $username | grep xplanets] }
    catch { exec kill -STOP $killwhat}
    .controls.button#2 configure -text Resume
    .controls.button#2 configure -command Resume-Planets
}

##########################################################################
# This procedure sends xplanets the "CONT" signal to resume it
##########################################################################
proc Resume-Planets { } {
    set username [exec whoami]
    catch { set killwhat [exec ps -u $username | grep xplanets] }
    catch { exec kill -CONT $killwhat}
    .controls.button#2 configure -text "Pause"
    .controls.button#2 configure -command Pause-Planets
}

##########################################################################
# This procedure begins the planets program running, and allows the
# user to pause and resume it.
# Xplanets reads in earth, venus, then mars.  0 is to print to the screen
# and 99999999 is used to make the program run indefinately
##########################################################################
proc RunPlanet { vmass emass mmass trail } {
    
    set GoForIt 1

    ##################################################
    # Let's check for non-numeric values in the fields
    ##################################################
    if {[regexp {[a-z,A-Z,$,!,@,#,%,^,&,*,(,),-,=,+,_,<,>,/,?,,,;,:,',",|,\,`,~]} $vmass]} {
	About "Venus' mass must be a\nnumeric value.\n" 90
	set GoForIt 0
    }

    if {[regexp {[a-z,A-Z,$,!,@,#,%,^,&,*,(,),-,=,+,_,<,>,/,?,,,;,:,',",|,\,`,~]} $emass]} {
	About "Earth's mass must be a\nnumeric value.\n" 90
	set GoForIt 0
    }

    if {[regexp {[a-z,A-Z,$,!,@,#,%,^,&,*,(,),-,=,+,_,<,>,/,?,,,;,:,',",|,\,`,~]} $mmass]} {
	About "Mars' mass must be a\nnumeric value.\n" 90
	set GoForIt 0
    }
    
    ##################################################
    # If everything is in order, let's start!
    ##################################################
    if {$GoForIt > 0} {
	set vmass [expr $vmass / 1.0]
	set emass [expr $emass / 1.0]
	set mmass [expr $mmass / 1.0]
	
	exec echo $trail > planet-input
	exec echo $emass >> planet-input
	exec echo $vmass >> planet-input
	exec echo $mmass >> planet-input
	# Plot on screen obviously
	exec echo 0 >> planet-input
	exec echo " " >> planet-input
	exec echo " " >> planet-input
	exec echo 999999999. >> planet-input
    
	exec xplanets < planet-input > /dev/null &
	
	toplevel .controls
	wm transient .controls .
	
	button .controls.button#1 \
		-command Stop-Planets \
		-text Stop
	
	button .controls.button#2 \
		-command {Pause-Planets} \
		-text "Pause" \
		-width 6
	grab .controls
	
	#####################################################################
	# Geometry management
	#####################################################################
	grid .controls.button#1 -in .controls   -row 2 -column 2  \
		-sticky w
	grid .controls.button#2 -in .controls   -row 2 -column 3  \
		-sticky e
	
	# Resize behavior management
	
	grid rowconfigure .controls 1 -weight 0 -minsize 30
	grid rowconfigure .controls 2 -weight 0 -minsize 30
	grid rowconfigure .controls 3 -weight 0 -minsize 30
	grid columnconfigure .controls 1 -weight 0 -minsize 30
	grid columnconfigure .controls 2 -weight 0 -minsize 74
	grid columnconfigure .controls 3 -weight 0 -minsize 45
	grid columnconfigure .controls 4 -weight 0 -minsize 30
    }
}
##########################################################################
# This procedure will display information about the planet clicked on
##########################################################################
proc PlanetInfo { w planet } {
    toplevel $w
#    wm transient $w .
    wm title $w "Planet Information"
    
    label $w.label#1 \
	    -text spam
    catch {
	$w.label#1 configure \
		-font -*-Helvetica-Bold-R-Normal-*-*-180-*-*-*-*-*-*
    }
    
    message $w.message#1 \
	    -borderwidth 1 \
	    -relief raised

    button $w.button#1 \
	    -command "destroy $w" \
	    -default active \
	    -text Dismiss

    ############################################################
    # Geometry management
    ############################################################
    grid $w.label#1 -in $w	-row 2 -column 2 
    grid $w.message#1 -in  $w	-row 4 -column 2  \
	    -sticky nesw
    grid $w.button#1 -in $w	-row 6 -column 2 
    
    # Resize behavior management
    
    grid rowconfigure $w 1 -weight 0 -minsize 20
    grid rowconfigure $w 2 -weight 0 -minsize 30
    grid rowconfigure $w 3 -weight 0 -minsize 8
    grid rowconfigure $w 4 -weight 0 -minsize 440
    grid rowconfigure $w 5 -weight 0 -minsize 9
    grid rowconfigure $w 6 -weight 0 -minsize 15
    grid rowconfigure $w 7 -weight 0 -minsize 25
    grid columnconfigure $w 1 -weight 0 -minsize 30
    grid columnconfigure $w 2 -weight 0 -minsize 485
    grid columnconfigure $w 3 -weight 0 -minsize 30

    if {[string compare $planet "venus"] == 0} {
	$w.label#1 configure -text "Venus Profile"

	$w.message#1 configure -text "
	Mass (kg).........................................................4.87 x 10^24
	Diameter (km)...................................................12104
	Mean density (kg/m^3) .....................................5250
	Escape velocity (m/sec)....................................10400
	
	Average distance from Sun (AU)............................0.723
	Rotation period (length of day in Earth days)..........243.0 (retrograde)
	Revolution period (length of year in Earth days).....224.7
	
	Obliquity (tilt of axis degrees)............................178
	Orbit inclination (degrees)..................................3.39
	Orbit eccentricity (deviation from circular).........0.007
	
	Mean surface temperature (K).........................726
	
	Visual geometric albedo (reflectivity)...............0.59
	Highest point on surface.............................Maxwell Montes
	                               (17 km above mean planetary radius)
	Atmospheric components...............................96% carbon dioxide,
                                                                                   3% nitrogen,
                                                                                   0.1% water vapor
	Surface materials....................................basaltic rock and
	                                                               altered materials

     For more information, see http://pds.jpl.nasa.gov/planets/welcome/venus.htm"
	
	
    } elseif {[string compare $planet "earth"] == 0} {
	$w.label#1 configure -text "Earth Profile"
	$w.message#1 configure -text "
	Mass (kg)...................................................5.98 x 10^24
	Diameter (km)............................................12756
	Mean density (kg/m^3) ...............................5520
	Escape velocity (m/sec)..............................11200
	
	Average distance from Sun (AU)...........................1
	Rotation period (length of day in Earth hours).........23.93
	Revolution period (length of year in Earth days).....365.26
	
	Obliquity (tilt of axis in degrees)........................23.4
	Orbit inclination (degrees)..................................0
	Orbit eccentricity (deviation from circular).........0.017
	
	Mean surface temperature (K).........................281
	Maximum surface temperature (K)...................310
	Minimum surface temperature (K)....................260
	
	Visual geometric albedo (reflectivity)...............0.39
	Highest point on surface.............................Mount Everest
	                                                (over 8 km above sea-level)
	Atmospheric components...............................78% nitrogen,
	                                                                     21% oxygen,
                                                                                   1% argon
	Surface materials...........................................basaltic and
                                                                                   granitic rock and
	                                                                     altered materials

 For more information, see http://pds.jpl.nasa.gov/planets/welcome/earth.htm"

    } elseif {[string compare $planet "mars"] == 0} {
	$w.label#1 configure -text "Mars Profile"
	$w.message#1 configure -text "
	Mass (kg)...................................................6.42 x 10^23
	Diameter (km).............................................6787
	Mean density (kg/m^3) ...............................3940
	Escape velocity (m/sec)..............................5000
	
	Average distance from Sun (AU)...........................1.524
	Rotation period (length of day in Earth days).........1.026
	Revolution period (length of year in Earth days).....686.98
	
	Obliquity (tilt of axis in degrees)........................25
	Orbit inclination (degrees)..................................1.85
	Orbit eccentricity (deviation from circular).........0.093
	
	Maximum surface temperature (K)......................310
	Minimum surface temperature (K).......................150
	
	Visual geometric albedo (reflectivity).................0.15
	Highest point on surface....................................Olympus Mons
                             	(about 24 km above surrounding lava plains)
	Atmospheric components...............................95% carbon dioxide,
                                                                                   3% nitrogen,
                                                                                   1.6% argon
	Surface materials....................................basaltic rock and
                                                                            altered materials

  For more information, see http://pds.jpl.nasa.gov/planets/welcome/mars.htm"
    }	
}

##########################################################################
# This procedure checks to make sure all three fields are filled in
# before enabling the "Start" button
##########################################################################
proc CheckArgs { venusmass earthmass marsmass} {

    if {[string compare $venusmass ""] == 1} {
        if {[string compare $earthmass ""] == 1} {
            if {[string compare $marsmass ""] == 1} {
                .button#2 configure -state normal
            } elseif {[string compare $marsmass ""] == 0} {
                .button#2 configure -state disabled
            }
        } elseif {[string compare $earthmass ""] == 0} {
            .button#2 configure -state disabled
        }
    } elseif {[string compare $venusmass ""] == 0} {
        .button#2 configure -state disabled
    }
}

##################################################################
# Make splash-screen
##################################################################
proc splash_screen { } {
    wm withdraw .
    toplevel .splash -borderwidth 4 -relief raised
    wm overrideredirect .splash 1

    after idle {
        update idletasks
        set xmax [winfo screenwidth .splash]
        set ymax [winfo screenheight .splash]
        set x0 [expr ($xmax-[winfo reqwidth .splash])/2]
        set y0 [expr ($ymax-[winfo reqheight .splash])/2]
        wm geometry .splash "+$x0+$y0"
    }

    label .splash.info \
            -text "http://www.lcse.umn.edu/specs"
    pack .splash.info -side bottom -fill x
    catch {.splash.info configure \
            -font -*-helvetica-medium-r-normal--*-100-*}
    
    set imh [image create photo -file images/lcse.gif]

    label .splash.icon -image $imh
    pack .splash.icon -side left -padx 8 -pady 8
    label .splash.title -text "Planetary Motion\nBy Chris White,\nPaul Woodward, and\nRich Solstad"
    pack .splash.title -fill x -padx 8 -pady 8
    catch {.splash.title configure \
            -font -*-helvetica-bold-o-normal--*-140-*}
    label .splash.status -text "Initializing Main Memory"
    pack .splash.status -fill x -pady 8
    catch {.splash.status configure \
            -font -*-helvetica-medium-r-normal--*-120-*}
    update
    exec sleep 2
    .splash.status configure -text "Initializing Hard Drive"
    update
    exec sleep 2
    .splash.status configure -text "Initializing Video Drivers"
    update
    exec sleep 2
    update
    wm withdraw .splash
    destroy .splash
#    wm deiconify .
}

##########################################################################
# Main routine to create main window
##########################################################################
proc interface_ui {root args} {

    set trails 0

    splash_screen

    # this treats "." as a special case
    if {$root == "."} {
	set base ""
    } else {
	set base $root
    }

    wm geometry $base. "+150+150"

    frame $base.frame#2
    
    frame $base.frame#4
    
    frame $base.frame#5
    
    label $base.label#1 \
	    -text {Plantary Motion Simulation}
    catch {
	$base.label#1 configure \
		-font -*-Helvetica-Bold-R-Normal-*-*-180-*-*-*-*-*-*
    }
    
    set venus [image create photo -file images/venglobe.gif]
    
    label $base.label#6 \
	    -cursor question_arrow \
	    -image $venus
    
    bind $base.label#6 <Button-1> {PlanetInfo .v venus}
    
    label $base.label#2 \
	    -text {Increase size of Venus by a factor of:}
    catch {
	$base.label#2 configure \
		-font -*-Helvetica-Bold-R-Normal-*-*-100-*-*-*-*-*-*
    }
    
    entry $base.entry#5 \
	    -textvariable venusmass \
	    -width 6

    bind $base.entry#5 <KeyRelease> {CheckArgs $venusmass $earthmass $marsmass}

    
    set earth [image create photo -file images/earglobe.gif]
    
    label $base.label#7 \
	    -cursor question_arrow \
	    -image $earth
    
    bind $base.label#7 <Button-1> {PlanetInfo .e earth}
    
    label $base.label#5 \
	    -text {Increase size of Earth by a factor of:}
    catch {
	$base.label#5 configure \
		-font -*-Helvetica-Bold-R-Normal-*-*-100-*-*-*-*-*-*
    }
    
    entry $base.entry#4 \
	    -textvariable earthmass \
	    -width 6
    bind $base.entry#4 <KeyRelease> {CheckArgs $venusmass $earthmass $marsmass}
    
    set mars [image create photo -file images/marglobe.gif]
    
    label $base.label#8 \
	    -cursor question_arrow \
	    -image $mars
    
    bind $base.label#8 <Button-1> {PlanetInfo .m mars}
    
    label $base.label#4 \
	    -text {Increase size of Mars by a factor of:}
    catch {
	$base.label#4 configure \
		-font -*-Helvetica-Bold-R-Normal-*-*-100-*-*-*-*-*-*
    }
    
    entry $base.entry#3 \
	    -textvariable marsmass \
	    -width 6
    bind $base.entry#3 <KeyRelease> {CheckArgs $venusmass $earthmass $marsmass}
    
    checkbutton $base.checkbutton#1 \
	    -text {Display Planetary Trails?} \
	    -variable trails

    button $base.button#3 \
	    -command exit \
	    -text Quit

    button $base.button#2 \
	    -command {RunPlanet $venusmass $earthmass $marsmass $trails} \
	    -state disabled \
	    -text Start
    
    menubutton $base.menu#1 \
            -text File \
            -menu $base.menu#1.menu
    set m [menu .menu#1.menu -tearoff 0]
    $m add command -label "Quit" -command {exit}
    
    menubutton $base.menu#2 \
	    -text Help \
            -menu $base.menu#2.menu
    
    set m2 [menu .menu#2.menu -tearoff 0]
    $m2 add command -label "About" -command {About "Planetary Motion\nVersion 0.5\n\nProgram written by: Paul Woodward\nEmail: paul@lcse.umn.edu\n\nAdditional modifications by: Rich Solstad\nEmail: rich@lcse.umn.edu\n\nGUI written by: Christopher White\nEmail: white@lcse.umn.edu\n\nCopyright 1998, All Rights Reserved" 255}

    ######################################################################
    # Geometry management
    ######################################################################
    
    grid $base.menu#1 -in $root     -row 1 -column 1
    grid $base.menu#2 -in $root     -row 1 -column 5
    grid $base.frame#2 -in $root	-row 4 -column 3  \
	    -ipady 8 \
	    -sticky w
    grid $base.frame#4 -in $root	-row 8 -column 3  \
	    -ipady 8 \
	    -sticky w
    grid $base.frame#5 -in $root	-row 6 -column 3  \
	    -ipady 8 \
	    -sticky w
    grid $base.label#1 -in $root	-row 2 -column 3 
    grid $base.label#6 -in $base.frame#2	-row 1 -column 1  \
	    -sticky nesw
    grid $base.label#2 -in $base.frame#2	-row 1 -column 3  \
	    -sticky w
    grid $base.entry#5 -in $base.frame#2	-row 1 -column 4  \
	    -sticky w
    grid $base.label#7 -in $base.frame#5	-row 1 -column 1  \
	    -sticky nesw
    grid $base.label#5 -in $base.frame#5	-row 1 -column 3  \
	    -sticky w
    grid $base.entry#4 -in $base.frame#5	-row 1 -column 4  \
	    -sticky e
    grid $base.label#8 -in $base.frame#4	-row 1 -column 1  \
	    -sticky nesw
    grid $base.label#4 -in $base.frame#4	-row 1 -column 3  \
	    -sticky w
    grid $base.entry#3 -in $base.frame#4	-row 1 -column 4  \
	    -sticky e
    grid $base.checkbutton#1 -in $root	-row 10 -column 3  \
	    -sticky e
    grid $base.button#3 -in $root	-row 12 -column 2 
    grid $base.button#2 -in $root	-row 12 -column 4 
    
    # Resize behavior management
    
    grid rowconfigure $base.frame#2 1 -weight 0 -minsize 30
    grid columnconfigure $base.frame#2 1 -weight 0 -minsize 48
    grid columnconfigure $base.frame#2 2 -weight 0 -minsize 30
    grid columnconfigure $base.frame#2 3 -weight 0 -minsize 19
    grid columnconfigure $base.frame#2 4 -weight 0 -minsize 52
    
    grid rowconfigure $base.frame#4 1 -weight 0 -minsize 30
    grid columnconfigure $base.frame#4 1 -weight 0 -minsize 49
    grid columnconfigure $base.frame#4 2 -weight 0 -minsize 30
    grid columnconfigure $base.frame#4 3 -weight 0 -minsize 79
    grid columnconfigure $base.frame#4 4 -weight 0 -minsize 34
    
    grid rowconfigure $root 1 -weight 0 -minsize 22
    grid rowconfigure $root 2 -weight 0 -minsize 30
    grid rowconfigure $root 3 -weight 0 -minsize 30
    grid rowconfigure $root 4 -weight 0 -minsize 30
    grid rowconfigure $root 5 -weight 0 -minsize 11
    grid rowconfigure $root 6 -weight 0 -minsize 30
    grid rowconfigure $root 7 -weight 0 -minsize 11
    grid rowconfigure $root 8 -weight 0 -minsize 30
    grid rowconfigure $root 9 -weight 0 -minsize 24
    grid rowconfigure $root 10 -weight 0 -minsize 30
    grid rowconfigure $root 11 -weight 0 -minsize 8
    grid rowconfigure $root 12 -weight 0 -minsize 30
    grid rowconfigure $root 13 -weight 0 -minsize 18
    grid columnconfigure $root 1 -weight 0 -minsize 18
    grid columnconfigure $root 2 -weight 0 -minsize 9
    grid columnconfigure $root 3 -weight 0 -minsize 2
    grid columnconfigure $root 4 -weight 0 -minsize 30
    grid columnconfigure $root 5 -weight 0 -minsize 30

    grid rowconfigure $base.frame#5 1 -weight 0 -minsize 30
    grid columnconfigure $base.frame#5 1 -weight 0 -minsize 47
    grid columnconfigure $base.frame#5 2 -weight 0 -minsize 32
    grid columnconfigure $base.frame#5 3 -weight 0 -minsize 25
    grid columnconfigure $base.frame#5 4 -weight 0 -minsize 32
    # additional interface code
    # end additional interface code
    wm deiconify .   
}

interface_ui .
